#! /usr/bin/python
# this is the first comment

import sys, os, optparse, numpy

class FFT():
    raw_file  = "data_d.dat"
    fft_file  = "data_freq_f.dat"

    def __init__(self, N, r_seed, central_freq, power):
        self.N         = N
        self.rand_seed = float(r_seed)
        self.cent_freq = float(central_freq) - 0.5
        self.power     = float(power)
        self.init_grid = numpy.zeros((self.N, self.N), dtype=complex)
        self.fft_grid  = None
        

        self.write_raw_data = False #True
        self.write_fft_data = False #True


    def writeData(self, file_name, grid):
        fout = open(file_name, "w")
        for i in range(self.N):
            for j in range(self.N):
                fout.write("%s %s %s %s\n" % (i, j, numpy.real(grid[i, j]), numpy.imag(grid[i, j])))
                
        fout.close()

    def genInitData(self):
        numpy.random.seed = self.rand_seed
        for i in range(self.N):
            for j in range(self.N):
                damping_factor = numpy.power(numpy.sqrt(numpy.power(i - self.cent_freq, 2) +
                                                        numpy.power(j - self.cent_freq, 2)), self.power)
                self.init_grid[i, j] = ((numpy.random.random() - 0.5) +
                                        (numpy.random.random() - 0.5)*1j)/damping_factor

        if self.write_raw_data:
            self.writeData(FFT.raw_file, self.init_grid)

    def performFFT(self):
        self.fft_grid = numpy.fft.fft2(self.init_grid)
        if self.write_fft_data:
            self.writeData(FFT.fft_file, self.fft_grid)

    def execute(self):
        self.genInitData()
        self.performFFT()

class GenPovFile():

    def __init__(self, size, grid_in, file_out, png_file):
        self.grid_points     = int(size)
        self.tot_points      = self.grid_points
        self.scalar_amplify  = 1000.0*(self.grid_points/128.0) #This value is optimized for 128^2 grids, but will be rescaled for any size of grid.
        self.min_y           = 1
        self.max_y           = -1
        self.max_x           = 0
        self.max_z           = 0
        self.color_map       = \
"""[0.0  color DarkGreen]
[0.2  color LimeGreen]
[0.4  color Brown]
[0.6  color SteelBlue]
[0.8  color Gray60]
[1.0  color White]"""
        
        self.nm_color_map    = \
"""[0.0  color Black]
[0.2  color Gray20]
[0.4  color Gray40]
[0.6  color Gray60]
[0.8  color Gray80]
[1.0  color White]"""

        self.scale           = 1.0
        self.scale_y         = 1.0
        self.grid_in         = grid_in
        self.Coords          = numpy.zeros((self.grid_points, self.grid_points), dtype=float)
        self.povrayfile      = file_out
        self.png_file        = png_file
        #self.color_map       = self.nm_color_map

    def findMinMaxZ(self):

        self.min_y = abs(self.grid_in[0, 0])
        self.max_y = abs(self.grid_in[0, 0])
        for i in range(self.grid_points):
            for j in range(self.grid_points):
                self.Coords[i, j] = abs(self.grid_in[i, j])
                if self.Coords[i, j] < self.min_y:
                    self.min_y = self.Coords[i, j]
                if self.max_y < self.Coords[i, j]:
                    self.max_y = self.Coords[i, j]

        self.scale_y = 1.0/(self.max_y - self.min_y)
        self.max_x = self.grid_points
        self.max_z = self.grid_points
        
    def drawTriangles(self):        
        outstring = """#include \"colors.inc\"
#include \"textures.inc\"
#include \"glass.inc\"
background{Black}"""

        outstring += """camera { angle 10
location <%s, %s, %s>
look_at <%s, %s, %s>}\n""" % (0, self.scalar_amplify, 0, 0, 0, 0)
        
        outstring += """light_source { <%s, %s, %s> color White shadowless}
#declare surfface = union{ \n""" % (0, self.scalar_amplify, 0)
        
        for i in range(self.grid_points-1):
            for j in range(self.grid_points-1):

                outstring += 'triangle{'
                outstring += '<%s, %s, %s>' % (str(i - self.max_x/2.0), 
                                               str(self.scale_y*(self.Coords[i, j] - self.min_y)),
                                               str(j - self.max_z/2.0))
                outstring += '<%s, %s, %s>' % (str(i + 1 - self.max_x/2.0),
                                               str(self.scale_y*(self.Coords[i + 1, j] - self.min_y)), 
                                               str(j - self.max_z/2.0))
                outstring += '<%s, %s, %s>\n' % (str(i - self.max_x/2.0), 
                                                 str(self.scale_y*(self.Coords[i, j + 1] - self.min_y)),
                                                 str(j + 1 - self.max_z/2.0))
                outstring += """pigment{gradient y
color_map {
%s
}}}\n""" % self.color_map
                
                outstring += 'triangle{'
                outstring += '<%s, %s, %s>' % (str(i + 1 - self.max_x/2.0), 
                                               str(self.scale_y*(self.Coords[i + 1, j] - self.min_y)), 
                                               str(j - self.max_z/2.0))
                outstring += '<%s, %s, %s>' % (str(i - self.max_x/2.0),
                                               str(self.scale_y*(self.Coords[i, j + 1] - self.min_y)),
                                               str(j + 1 - self.max_z/2.0))
                outstring += '<%s, %s, %s>\n' % (str(i + 1 - self.max_x/2.0),
                                                 str(self.scale_y*(self.Coords[i + 1, j + 1] - self.min_y)),
                                                 str(j + 1 - self.max_z/2.0))
                outstring += """pigment{gradient y
color_map {
%s
}}}\n""" % self.color_map
        outstring +='}\n'
        outstring += 'object{surfface scale<1.33, 1, 1>}\n'
        f  = open(self.povrayfile , 'w')
        f.write(outstring)
        f.close()

    def execute(self):
        self.findMinMaxZ()
        self.drawTriangles()
        #os.system('povray %s +O./%s' % (self.povrayfile, self.png_file))

def main():
    parser = optparse.OptionParser()
    parser.add_option('--size', metavar="SIZE", action="store", dest="size", help="Specify the size of the grid", default = "64", type = int)
    parser.add_option('--seed', metavar="SEED", action="store", dest="seed", help="The seed for the random number generator", default="0", type = float)
    parser.add_option('--central_freq', metavar="C_FREQ", action="store", dest="c_freq", help="Central frequency from which the Fourier modes are decaying away", default="0", type = float)
    parser.add_option('--power', metavar="POW", action="store", dest="pow", help="Power of the distance suppresing the modes away from the central frequency", default="1", type = float)
    parser.add_option('--pov_file', metavar="POVFILE", action="store", dest="pov_file", help="File to be fed to POV-RAY for rendering", default="surface.pov", type = str)
    parser.add_option('--png_file', metavar="PNGFILE", action="store", dest="png_file", help="Final output PNG file", default="surface.png", type = str)
    
    (args, remainder) = parser.parse_args()    
    X = FFT(args.size, args.seed, args.c_freq, args.pow)
    X.execute()

    Y = GenPovFile(args.size, X.fft_grid, args.pov_file, args.png_file)
    Y.execute()

if __name__ == "__main__":
    main()
